#
# Copyright (c) 2007, Brian Frank and Andy Frank
# Licensed under the Academic Free License version 3.0
#
# History:
#   13 Jun 07  Andy Frank  Creation
#

require 'net/ftp'
require 'adm/env.rb'

#
# Deploy the development web site to fandev.org
#
class DeployWWW < Env

  # init
  def initialize
    super
    @username = ""
    @password = ""
    @fandoc_src = File.join(@fan_home, "doc")
    @fandoc_target = "test"
    @count = 0
  end

  # main
  def main()
    print "Username: "
    @username = $stdin.gets.chomp!

    print "Password: "
    @password = $stdin.gets.chomp!

    if (ARGV.include?("clean"))
      clean
    elsif (ARGV.include?("full"))
      clean
      deploy
    else
      deploy
    end
    puts "\n*** Success! ****"
  end

  # clean
  def clean()
    puts "[clean]"
    Net::FTP.open('ftp.fandev.org') do |ftp|
      ftp.login(@username, @password)
      ftp.chdir('public_html')
      ftp.nlst.each do |f|
        cleanDirs(ftp, f, "/#{@fandoc_target}") if f == @fandoc_target
      end
    end
  end

  # recurse down parent dir removing all
  # files and sub directories
  def cleanDirs(ftp, parent, path)
    ftp.chdir(parent)
    ftp.nlst.each do |f|
      if (f != '.' && f != '..')
        if (!f.include?('.')) # <-- major hack!
          cleanDirs(ftp, f, "#{path}/#{f}")
        else
          puts "  Delete  #{path}/#{f}"
          ftp.delete(f)
        end
      end
    end
    ftp.chdir("..")
    puts "  Rmdir   #{path}"
    ftp.rmdir(parent)
  end

  # deploy
  def deploy()
    puts "[deploy]"
    Net::FTP.open('ftp.fandev.org') do |ftp|
      ftp.login(@username, @password)
      ftp.chdir('public_html')
      putFiles(ftp, @fandoc_src, @fandoc_target, "/#{@fandoc_target}")
      puts "\n#{@count} files transferred"
    end
  end

  # recurse down the parent dir, putting all files
  # onto the server that have been modified
  def putFiles(ftp, srcdir, targetdir, path)
    begin
      # verify targetdir exists first
      ftp.chdir(targetdir)
    rescue
      # if it does not exist, create it
      puts "  Mkdir  #{path}"
      ftp.mkdir(targetdir)
      ftp.chdir(targetdir)
    end
    curr = ftp.nlst
    Dir.foreach(srcdir) do |fn|
      if (fn != '.' && fn != '..')
        f = File.join(srcdir, fn)
        if (File.directory?(f))
          putFiles(ftp, f, fn, "#{path}/#{fn}")
        elsif (curr.include?(fn))
          puts "  Skip  #{path}/#{fn}"
        else
          puts "  Put  #{path}/#{fn}"
          ftp.put(f)
          @count = @count + 1
        end
      end
    end
    ftp.chdir("..")
  end

end

# script
DeployWWW.new.main
