//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   6 Nov 06  Brian Frank  Creation
//

using build

**
** buildall.fan
**
** This is the top level build script for building Fan - it routes
** build functions to two sub-scripts buildboot and buildpods.  This
** top level script itself and buildboot.fan must be hosted by a
** substitute runtime defined in sys.props used to bootstrap this
** development environment to a self-buildable state.
**
class Build : BuildGroup
{

//////////////////////////////////////////////////////////////////////////
// Setup
//////////////////////////////////////////////////////////////////////////

  override Void setup()
  {
    childrenScripts =
    [
      `buildboot.fan`,
      `buildpods.fan`,
    ]
  }

//////////////////////////////////////////////////////////////////////////
// Targets
//////////////////////////////////////////////////////////////////////////

  override Target[] targets()
  {
    mine :=
    [
      Target.make(this, "compile",    "run compile on all pods",  &compile),
      Target.make(this, "clean",      "run clean on all pods",    &clean),
      Target.make(this, "full",       "run full on all pods",     &full),
      Target.make(this, "test",       "run test on all podspods", &test),
      Target.make(this, "doc",        "build fandoc HTML docs",   &doc),
      Target.make(this, "superclean", "delete lib dir",           &superclean),
      Target.make(this, "dist",       "full build w/ doc",        &dist),
    ]
    return mine.addAll(super.targets)
  }

//////////////////////////////////////////////////////////////////////////
// Compile
//////////////////////////////////////////////////////////////////////////

  Void compile()
  {
    spawnOnChildren("compile")
  }

//////////////////////////////////////////////////////////////////////////
// Clean
//////////////////////////////////////////////////////////////////////////

  Void clean()
  {
    runOnChildren("clean")
  }

//////////////////////////////////////////////////////////////////////////
// Full
//////////////////////////////////////////////////////////////////////////

  Void full()
  {
    runOnChildren("clean")
    spawnOnChildren("full")
  }

//////////////////////////////////////////////////////////////////////////
// Test
//////////////////////////////////////////////////////////////////////////

  Void test()
  {
    Exec.make(this, [binDir+`fant.exe`.toStr, "*"]).run
  }

//////////////////////////////////////////////////////////////////////////
// Doc
//////////////////////////////////////////////////////////////////////////

  Void doc()
  {
    fanExe := (binDir+`fan.exe`).toStr
    allBuildPodScripts.each |BuildPod script|
    {
      if (script.podName == "sysTest") return
      Exec.make(this, [fanExe, "docCompiler", script.podName]).run
    }

    Exec.make(this, [fanExe, "docCompiler", "-topindex"]).run
  }

//////////////////////////////////////////////////////////////////////////
// Superclean
//////////////////////////////////////////////////////////////////////////

  Void superclean()
  {
    Delete.make(this, libFanDir).run
    Delete.make(this, libJavaDir).run
  }

//////////////////////////////////////////////////////////////////////////
// Dist
//////////////////////////////////////////////////////////////////////////

  Void dist()
  {
    superclean
    full
    doc
    test
    deleteNonDist
  }

  private Void deleteNonDist()
  {
    Delete.make(this, devHomeDir+`tmp/`).run
    Delete.make(this, devHomeDir+`src/jfan/temp/`).run

    libJavaDir.list.each |File f|
    {
      if (f.name != "sys.jar")
        Delete.make(this, f).run
    }
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  BuildPod[] allBuildPodScripts(BuildScript[] children := this.children)
  {
    // recursively find all the BuildPod scripts in
    // my children scripts and their descendents
    acc := BuildPod[,]
    children.each |BuildScript child|
    {
      if (child is BuildGroup)
      {
        group := child as BuildGroup
        acc.addAll((BuildPod[])group.children.findType(BuildPod.type))
        acc.addAll(allBuildPodScripts(group.children))
      }
    }
    return acc
  }

}
