//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Nov 07  Brian Frank  Creation
//

**
** Main is the main entry point for the Fan daemon.
**
class Main
{

//////////////////////////////////////////////////////////////////////////
// Run
//////////////////////////////////////////////////////////////////////////

  **
  ** Main entry point for compiler.
  **
  Int run(Str[] args)
  {
    // process args
    if (!parseArgs(args)) return 0

    // resolve app config file
    file := Sys.homeDir + "fand/$appName/fand.fog".toUri
    if (!file.exists)
    {
      println("ERROR: File not found: $file")
      return 1
    }

    // deserialize
    App app
    try
    {
      app = file.readObj
    }
    catch (Err e)
    {
      println("ERROR: Cannot load: $file")
      e.trace
      return 2
    }

    echo("Loaded app!")
    Sys.out.writeObj(app).flush
    return 0
  }

  **
  ** Process command line args and return false if we should exit.
  **
  Bool parseArgs(Str[] args)
  {
    if (args.isEmpty)
    {
      help
      return false
    }

    for (i:=0; i<args.size; ++i)
    {
      a := args[i]
      if (a.isEmpty) continue
      if (a == "-help" || a == "-h" || a == "-?")
      {
        help
        return false
      }
      else if (a == "-version")
      {
        version
        return false
      }
      else if (a[0] == '-')
      {
        println("WARNING: Unknown option '$a'")
      }
      else
      {
        if (appName == null)
          appName = a
        else
          println("WARNING: Argument ignored '$a'")
      }
    }

    // not enough arguments
    if (appName == null)
    {
      println("ERROR: not enough arguments")
      help
      return false
    }

    return true
  }

  **
  ** Dump help usage.
  **
  Void help()
  {
    println("Fan Daemon v${type.pod.version}")
    println("Usage:")
    println("  fand [options] <appName>")
    println("Options:")
    println("  -help, -h, -?  print usage help")
    println("  -version       print version information")
  }

  **
  ** Dump version.
  **
  Void version()
  {
    println("Fan Daemon v${type.pod.version}")
    println("Copyright (c) 2007, Brian Frank and Andy Frank")
    println("Licensed under the Academic Free License version 3.0")
  }

  **
  ** Print to stdout
  **
  Void println(Str s)
  {
    echo(s)
  }

//////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////

  static Void main()
  {
    Sys.exit(make.run(Sys.args))
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  Str appName

}