//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   24 Nov 07  Brian Frank  Creation
//
package fan.sys;

import java.util.*;

/**
 * Resource models an object in the Uri namespace.  Resources
 * delegate all methods to a ResourcePeer which implements the
 * functionality in Java code.  Binding between a Resource and
 * its ResourcePeer occur in the ResourcePeer constructor and
 * last for the lifetime of both instances.
 */
public abstract class Resource
  extends FanObj
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public static void make$(Resource self, Uri uri)
  {
    if (uri == null) throw ArgErr.make("Null uri").val;
    self.uri = uri;
  }

  public Resource(Uri uri) { make$(this, uri); }
  public Resource() {}

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  public Str toStr() { return uri.toStr(); }

  public Type type() { return Sys.ResourceType; }

//////////////////////////////////////////////////////////////////////////
// Uri
//////////////////////////////////////////////////////////////////////////

  public final Uri uri() { return uri; }

  public final Bool isDir() { return uri.isDir();   }

  public final List path() { return uri.path(); }

  public final Str pathStr() { return uri.pathStr(); }

  public final Str name() { return uri.name(); }

  public final Str basename() { return uri.basename(); }

  public final Str ext() { return uri.ext(); }

//////////////////////////////////////////////////////////////////////////
// Resource Overrides
//////////////////////////////////////////////////////////////////////////

  public Bool exists()
  {
    throw UnsupportedErr.make(type() + ".exists").val;
  }

  public Obj obj()
  {
    return null;
  }

  public void obj(Obj obj)
  {
    throw UnsupportedErr.make(type() + ".obj").val;
  }

  public List list()
  {
    return emptyList;
  }

  public void refresh()
  {
    throw UnsupportedErr.make(type() + ".refresh").val;
  }

  public Resource create()
  {
    throw UnsupportedErr.make(type() + ".create").val;
  }

  public Resource add(Obj obj)
  {
    throw UnsupportedErr.make(type() + ".add").val;
  }

  public void save()
  {
    throw UnsupportedErr.make(type() + ".save").val;
  }

  public void delete()
  {
    throw UnsupportedErr.make(type() + ".delete").val;
  }

  public Resource resolve(Uri uri, Int startDepth)
  {
    return null;
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  /**
   * Make a thread-safe copy of the specified object.
   * If it is immutable, then just return it; otherwise
   * we make a serialized copy.
   */
  public static Obj safe(Obj obj)
  {
    if (obj == null) return null;
    if (obj.isImmutable().val) return obj;
    Buf buf = Buf.make(256);
    buf.out.writeObj(obj);
    buf.flip();
    return buf.in.readObj();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public static final List emptyList = new List(Sys.ResourceType).ro();

  Uri uri;
}