//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jan 06  Andy Frank  Creation
//

using System;
using System.IO;
using System.Reflection;

namespace Fanx.Tools
{
  public abstract class Tool
  {

  //////////////////////////////////////////////////////////////////////////
  // Fan Assembly Loading
  //////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Register the assembly resolver.
    /// </summary>
    protected Tool()
    {
      AppDomain domain = AppDomain.CurrentDomain;
      domain.AssemblyResolve += new ResolveEventHandler(resolveAssembly);
    }

    /// <summary>
    /// Get the assembly name from the specified fullName.  The fullName
    /// has the following format:
    ///
    /// <name>, Version=<major.minor.build.patch>, Culture=<...>, PublicKeyToken=<...>
    ///
    /// </summary>
    public static string getAssemblyName(string fullName)
    {
      int sep = fullName.IndexOf(',');
      if (sep == -1)
        return fullName;
      else
        return fullName.Substring(0, sep);
    }

    /// <summary>
    /// Resolve an assembly that cannot be found in
    /// the current AppDomain.
    /// </summary>
    public static Assembly resolveAssembly(object sender, ResolveEventArgs args)
    {
      string asmName = getAssemblyName(args.Name);

      AppDomain domain = AppDomain.CurrentDomain;
      string homeDir = Environment.GetEnvironmentVariable("fan_home");
      if (homeDir == null)
        throw new Exception("Missing fan_home environment variable.");
      if (!Directory.Exists(homeDir))
        throw new Exception("Invalid fan_home directory: " + homeDir);

      // check if already loaded
      Assembly[] current = domain.GetAssemblies();
      for (int i=0; i<current.Length; i++)
        if (asmName == current[i].GetName().Name)
          return current[i];

      // otherwise load it from disk
      string libDir = Path.Combine(homeDir, "lib/net");
      string dll = Path.Combine(libDir, asmName + ".dll");

      FileInfo f = new FileInfo(dll);
      if (!f.Exists)
      {
        // check tmp dir
        string tmpDir = Path.Combine(homeDir, "lib/tmp");
        dll = Path.Combine(tmpDir, asmName + ".dll");
        f = new FileInfo(dll);
      }
      if (!f.Exists) return null;
      BinaryReader fIn = new BinaryReader(f.OpenRead());
      byte[] asm = fIn.ReadBytes((int)f.Length);
      fIn.Close();
      if (asm.Length != f.Length)
        throw new Exception("Could not read " + dll + ": " + asm.Length + " != " + f.Length);

      Assembly result = domain.Load(asm);
      return result;
    }

  }
}