//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   30 Jun 07  Brian Frank  Creation
//

**
** Connection manages a logical connection to an SQL database.
**
** Refer to the [Sql docs]`docLib::Sql` for more details.
**
class Connection
{

//////////////////////////////////////////////////////////////////////////
// Lifecycle
//////////////////////////////////////////////////////////////////////////

  **
  ** Open a connection to a SQL database.  Throw 'Err' on failure.
  **
  ** In a Java platform, the connection is opened using JDBC.  The
  ** database Str specifies the 'java.sql.Connection' URL.  The JDBC
  ** driver class should be preloaded - the simplest mechanism is to
  ** add it to "sys.props".
  **
  static native Connection open(Str database, Str username, Str password)

  **
  ** Internal constructor.
  **
  internal new make() {}

  **
  ** Return if this connection has been closed.
  **
  native Bool isClosed()

  **
  ** Close the connection.  This method is guaranteed to never throw
  ** an 'Err'.  Return true if the connection was closed successfully
  ** or 'false' if closed abnormally.
  **
  native Bool close()

//////////////////////////////////////////////////////////////////////////
// Database metadata
//////////////////////////////////////////////////////////////////////////

  **
  ** List the tables in the database.  Returns a list of the
  ** table names.
  **
  native Str[] tables()

  **
  ** Get a default row instance for the specified table.  The
  ** result has a field for each table column.
  **
  native Obj tableRow(Str tableName)

//////////////////////////////////////////////////////////////////////////
// Statements
//////////////////////////////////////////////////////////////////////////

  **
  ** Execute an SQL query which returns a relational table as a
  ** result.  If 'of' is 'null', the result is returned as a 'List'
  ** of 'Rows'.  The 'Cols' are available from 'List.of.fields' or
  ** on 'type.fields' of each row instance.  If 'of' is specified,
  ** then the result is returned as a 'List' of that type.  Columns
  ** are mapped to the type's fields.  If a column cannot be mapped
  ** to a field, then an 'SqlErr' is thrown.
  **
  native Obj[] query(Str sql, Type of := null)

  **
  ** Execute an SQL query.  For each row in the result, invoke
  ** the specified function 'c'.  If 'of' is null, the 'Obj'
  ** passed to the 'each' function will be of type 'Row'.
  ** Otherwise the 'of' type specifies the row type passed
  ** to the 'each' function.
  **
  native Void queryEach(Str sql, Type of, |Obj row| each)
  
  **
  ** Execute a SQL statement and if applicable return
  ** the number of rows modified.
  **
  native Int execute(Str sql)
  
  **
  ** Prepare a statement for use later with this connection.
  **
  Statement prepare(Str sql)
  {
    return Statement.make(this, sql).prepare()
  }

//////////////////////////////////////////////////////////////////////////
// Transactions
//////////////////////////////////////////////////////////////////////////

  **
  ** If auto commit is 'true', then each statement is executed
  ** immediately.  Otherwise statements are executed inside a
  ** transaction which is terminated by a call to 'commit'
  ** or 'rollback'.  Auto commit defaults to 'true'.
  **
  Bool autoCommit
  {
    get { return getAutoCommit }
    set { setAutoCommit(val) }
  }
  private native Bool getAutoCommit()
  private native Void setAutoCommit(Bool b)


  **
  ** Commit all the changes made inside the current transaction.
  **
  native Void commit()

  **
  ** Undo any changes made inside the current transaction.
  **
  native Void rollback()

}