//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Apr 06  Brian Frank  Creation
//

**
** EnumTest
**
class EnumTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Type
//////////////////////////////////////////////////////////////////////////

  Void testType()
  {
    verifyEq(EnumAbc.type.signature, "sysTest::EnumAbc")
    verifyEq(EnumAbc.type.base, Enum.type)
    verifyEq(EnumAbc.type.isEnum, true)
    verifyEq(EnumAbc.type.isClass, false)
    verifyEq(EnumAbc.type.isMixin, false)
    verifyEq(Obj.type.isEnum, false)
    verifyEq(Str.type.isEnum, false)

    verifySame(EnumAbc.A.type, EnumAbc.type)

    verify(EnumAbc.A is EnumAbc)
    verify(EnumAbc.A is Enum)
    verify(EnumAbc.A is Obj)
    verifyFalse((Obj)EnumAbc.A is Int)
  }

//////////////////////////////////////////////////////////////////////////
// Equals
//////////////////////////////////////////////////////////////////////////

  Void testEquals()
  {
    verify(EnumAbc.A == EnumAbc.A)
    verify(EnumAbc.A === EnumAbc.A)
    verify(EnumAbc.A != EnumAbc.B)
    verify(EnumAbc.A != EnumAbc.C)
    verify(EnumAbc.B != EnumAbc.C)
  }

//////////////////////////////////////////////////////////////////////////
// Compare
//////////////////////////////////////////////////////////////////////////

  Void testCompare()
  {
    verify(EnumAbc.A <= EnumAbc.A)
    verify(EnumAbc.A < EnumAbc.B)
    verify(EnumAbc.B < EnumAbc.C)
    verifyFalse(EnumAbc.C < EnumAbc.B)
    verifyFalse(EnumAbc.B < EnumAbc.A)
    verify(EnumAbc.B >= EnumAbc.B)
    verifyFalse(EnumAbc.B > EnumAbc.B)
    verify(EnumAbc.B > EnumAbc.A)

    verifyEq([EnumAbc.B, EnumAbc.C, EnumAbc.A].sort, [EnumAbc.A, EnumAbc.B, EnumAbc.C])
  }

//////////////////////////////////////////////////////////////////////////
// Ordinal
//////////////////////////////////////////////////////////////////////////

  Void testOrdinals()
  {
    verifyEq(EnumAbc.A.ordinal, 0)
    verifyEq(EnumAbc.B.ordinal, 1)
    verifyEq(EnumAbc.C.ordinal, 2)
  }

//////////////////////////////////////////////////////////////////////////
// Names
//////////////////////////////////////////////////////////////////////////

  Void testNames()
  {
    verifyEq(EnumAbc.A.name, "A")
    verifyEq(EnumAbc.B.name, "B")
    verifyEq(EnumAbc.C.name, "C")
  }

//////////////////////////////////////////////////////////////////////////
// To Str
//////////////////////////////////////////////////////////////////////////

  Void testToStr()
  {
    verifyEq(EnumAbc.A.toStr, "A")
    verifyEq(EnumAbc.B.toStr, "B")
    verifyEq(EnumAbc.C.toStr, "C")
  }

//////////////////////////////////////////////////////////////////////////
// Values
//////////////////////////////////////////////////////////////////////////

  Void testValues()
  {
    verifyEq(EnumAbc.values, [EnumAbc.A, EnumAbc.B, EnumAbc.C])
    verifyEq(EnumAbc.values.isRO, true)
  }

//////////////////////////////////////////////////////////////////////////
// Parse
//////////////////////////////////////////////////////////////////////////

  Void testParse()
  {
    verifySame(Weekday.fromStr("mon"), Weekday.mon)
    verifySame(Weekday.fromStr("xxx", false), null)
    verifyErr(ParseErr.type) |,| { Weekday.fromStr("xxx") }

    verifySame(Month.fromStr("apr"), Month.apr)
    verifySame(Month.fromStr("xxx", false), null)
    verifyErr(ParseErr.type) |,| { Month.fromStr("xxx") }

    verifySame(EnumAbc.fromStr("A"), EnumAbc.A)
    verifySame(EnumAbc.fromStr("B"), EnumAbc.B)
    verifySame(EnumAbc.fromStr("C"), EnumAbc.C)
    verifySame(EnumAbc.fromStr("values", false), null)
    verifySame(EnumAbc.fromStr("first", false), null)
    verifySame(EnumAbc.fromStr("foobar", false), null)

    verifySame(Suits.fromStr("clubs"), Suits.clubs)
    verifySame(Suits.fromStr("colors", false), null)

    verifyErr(ParseErr.type) |,| { EnumAbc.fromStr("values") }
    verifyErr(ParseErr.type) |,| { EnumAbc.fromStr("first", true) }
    verifyErr(ParseErr.type) |,| { EnumAbc.fromStr("foo") }
  }

//////////////////////////////////////////////////////////////////////////
// Additional Methods
//////////////////////////////////////////////////////////////////////////

  Void testAdditionalMethods()
  {
    verifyEq(EnumAbc.A.negOrdinal, 0)
    verifyEq(EnumAbc.B.negOrdinal, -1)
    verifyEq(EnumAbc.C.negOrdinal, -2)
  }

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  Void testCtor()
  {
    verifyEq(Suits.values, [Suits.clubs, Suits.diamonds, Suits.hearts, Suits.spades])
    verifyEq(Suits.clubs.ordinal,    0)
    verifyEq(Suits.diamonds.ordinal, 1)
    verifyEq(Suits.hearts.ordinal,   2)
    verifyEq(Suits.spades.ordinal,   3)

    verifyEq(Suits.clubs.color,    "black")
    verifyEq(Suits.diamonds.color, "red")
    verifyEq(Suits.hearts.color,   "red")
    verifyEq(Suits.spades.color,   "black")
  }

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  Void testReflection()
  {
    // enum has fields
    verify(EnumAbc.type.slot("A").isField)
    verifyEq(EnumAbc.type.field("A").name, "A")
    verifyEq(EnumAbc.type.field("A")->getter, null)
    verifyEq(EnumAbc.type.field("A")->setter, null)

    // get
    verifyEq(EnumAbc.type.field("A").get(null), EnumAbc.A)
    verifyEq(EnumAbc.type.field("B")[null], EnumAbc.B)
  }

//////////////////////////////////////////////////////////////////////////
// Facets
//////////////////////////////////////////////////////////////////////////

  Void testFacets()
  {
    verifyEq(EnumAbc.type.facet("simple"), true)
    verifyEq(EnumAbc.type.facets, Str:Obj["simple":true])

    verifyEq(Suits.type.facet("simple"), true)
    verifyEq(Suits.type.facet("x"), "y")
    verifyEq(Suits.type.facets, Str:Obj["simple":true, "x":"y"])
  }

}

internal enum EnumAbc
{
  A, B, C

  Int negOrdinal() { return -ordinal }

  static const EnumAbc first := A
}

@x="y"
enum Suits
{
  clubs("black"),
  diamonds("red"),
  hearts("red"),
  spades("black")

  private new make(Str color) { this.color = color; }

  const Str color;
}

enum Foo
{
  one,
  two,
  three
}