//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   31 Jul 07  Andy Frank  Creation
//

**
** Page is a Weblet designed to render Widgets.
**
abstract class Page : Weblet
{

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  **
  ** Render the markup for this widget. This method *must*
  ** use the `headElem` and `bodyElem` fields for rendering
  ** the markup in order to produce a valid HTML file.
  ** Never use 'res.out' directly.
  **
  abstract Void render()

  **
  ** Handle a form submission.
  **
  virtual Void submit()
  {
  }

//////////////////////////////////////////////////////////////////////////
// Weblet
//////////////////////////////////////////////////////////////////////////

  override Void get()
  {
    try
    {
      head := Buf.make
      body := Buf.make

      headElem = WebOutStream.make(head.out)
      bodyElem = WebOutStream.make(body.out)

      // cache locals
      Thread.locals["web.widget.head"] = headElem
      Thread.locals["web.widget.body"] = bodyElem

      // render the page
      render

      // finish the page
      // TODO - get encoding from OutStream
      res.headers["Content-Type"] = "text/html; charset=UTF-8"
      res.headers["Content-Encoding"] = "UTF-8"
      res.out.prolog
      res.out.docType
      res.out.html
      res.out.head
      res.out.writeBuf(head.flip)
      res.out.headEnd
      res.out.body
      res.out.writeBuf(body.flip)
      res.out.bodyEnd
      res.out.htmlEnd
    }
    finally
    {
      // remove locals
      Thread.locals.remove("web.widget.head")
      Thread.locals.remove("web.widget.body")
    }
  }

  override Void post()
  {
    isForm := req.headers["Content-Type"] == "application/x-www-form-urlencoded"
    action := req.uri.query["action"]

    if (action != null)
    {
      this.trap(action, [,])
    }
    else if (isForm)
    {
      submit
    }
    else res.sendError(404)
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  **
  ** The buffered WebOutStream for the <head> element.
  **
  readonly WebOutStream headElem

  **
  ** The buffered WebOutStream for the <body> element.
  **
  readonly WebOutStream bodyElem

}