//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Mar 06  Andy Frank  Creation
//

**
** WebReq encapsulates a web request.
**
abstract class WebReq
{

//////////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////////

  **
  ** Get the WebService managing the request.
  **
  abstract WebService service()

  **
  ** The HTTP request method in uppercase. Example: GET, POST, PUT.
  **
  abstract Str method()

  **
  ** The HTTP version of the request.
  **
  abstract Version version()

  **
  ** The request URI.
  **
  ** Examples:
  **   http://www.foo.com/a/b/c
  **   http://www.foo.com/a?q=bar
  **
  abstract Uri uri()

  **
  ** The prefixUri is the portion of the request URI controlled
  ** by the web server.  A prefixUri always ends with a trailing
  ** slash.
  **
  **   uri == prefixUri + suffixUri
  **
  ** Examples:
  **         uri: http://www.foo.com/a/b/c
  **   prefixUri: http://www.foo.com/a/
  **   suffixUri: b/c
  **
  **         uri: http://www.foo.com/a/b/c
  **   prefixUri: http://www.foo.com/
  **   suffixUri: a/b/c
  **
  **         uri: http://www.foo.com/a/b/c?q=bar
  **   prefixUri: http://www.foo.com/a/b/
  **   suffixUri: c?q=bar
  **
  abstract Uri prefixUri()

  **
  ** The suffixUri is the portion of the request URI not
  ** controlled by the web server.  The suffixUri will always
  ** starts after the trailing slash of the prefixUri.
  **
  **   uri == prefixUri + suffixUri
  **
  ** Examples:
  **         uri: http://www.foo.com/a/b/c
  **   prefixUri: http://www.foo.com/a/
  **   suffixUri: b/c
  **
  **         uri: http://www.foo.com/a/b/c
  **   prefixUri: http://www.foo.com/
  **   suffixUri: a/b/c
  **
  **         uri: http://www.foo.com/a/b/c?q=bar
  **   prefixUri: http://www.foo.com/a/b/
  **   suffixUri: c?q=bar
  **
  abstract Uri suffixUri()

  **
  ** Map of HTTP request headers.  This Map is readonly
  ** and case insenstive (see `sys::Map.caseInsensitive`).
  **
  abstract Str:Str headers()

  **
  ** The UserAgent for this request or null if the
  ** "User-Agent" header was not specified in the request.
  **
  abstract UserAgent userAgent()

  **
  ** Get the key/value pairs of the form data.  If the request
  ** content type is "application/x-www-form-urlencoded", then the
  ** first time this method is called the request content is read
  ** and parsed using `sys::Uri.decodeQuery`.  If the content
  ** type is not "application/x-www-form-urlencoded" this method
  ** returns null.
  **
  abstract Str:Str form()

  **
  ** The InStream for this request.
  **
  abstract InStream in()

  **
  ** The Resource resolved by the `uri`.
  **
  Resource resource { internal set; }

  **
  ** Stash allows you to stash objects on the WebReq object
  ** in order to pass data b/w Weblets on the same request.
  **
  abstract Str:Obj stash()

}